@echo -off
cls 0 15
echo " "
echo " *******************************************************************"
echo " *                 Kontron - Product Center Boards                 *"
echo " *                       BIOS Update script                        *"
echo " *******************************************************************"

# define variables for applications
set -v KoFind ".\EFI\Findstr.efi" > nul
set -v AFU ".\Afu\AfuEfix64.efi" > nul
set -v AFUParams "^^/B ^/P ^/N ^/K:A4FB2BAACD9847ca86A1C333225764DC ^/RLC:F" > nul
set -v FWUpdLcl ".\FWUpdLcl\FWUpdLcl.efi" > nul
set -v FWUpdLclParams "^^-Y ^-F" > nul

# clean up files and variables
rm smbios.out > nul
rm afu.out > nul
set -d FlashFile > nul
set -d product > nul
set -d FindStr > nul

# Read SMBIOS type 2 (Baseboard) to get the running board product type
smbiosview -t 2 >a smbios.out
%KoFind% /M /DK "ProductName*K*-??" smbios.out > nul
set -v product %FindStr% > nul
set -d FindStr > nul

# Read SMBIOS type 0 (BIOS) to get the running BIOS version
smbiosview -t 0 >>a smbios.out
%KoFind% /M /I /D".?? " /E" for " "BiosVersion*R?.*.*" smbios.out > nul
set -v BIOSV %FindStr% > nul
set -d FindStr > nul

# If this script was called with an argument, assume it's the BIOS flash filename
# Continue without searching for a suitable file
set -v FlashFile "0.0.0" > nul
if "%1" neq "" then
  set -v FlashFile %1 > nul
endif
if %FlashFile% eq "0.0.0" then
  # Search for suitable flash file in script directory, with highest revision
  for %f in K%product%*.bin
    if exists "%f" then
      if /i /s "%f" gt "%FlashFile%" then
        set -v FlashFile "%f" > nul
      endif
    endif
  endfor
endif

# Check if suitable flash BIN file was found
if "%FlashFile%" eq "0.0.0" then
  goto ERR_FILE
endif

# Make filename relative
%KoFind% /M /I /D"%cwd%\" "*" /S"%FlashFile%" > nul
set -v FlashFile %FindStr%

# BIOS version from Filename
%KoFind% /M /I /D"K%product%." "R?.*.*" /E".BIN" /S"%FlashFile%" > nul
set -v BIOS_TARGET_VERSION %FindStr% > nul
set -d FindStr > nul

# Check BIOS Version
if %BIOSV% == %BIOS_TARGET_VERSION% then
  echo " "
  echo " > BIOS for motherboard  : K%product%"
  echo " > BIOS version installed: %BIOSV%"
  echo " > BIOS version of file  : %BIOS_TARGET_VERSION%"
  echo " "
  echo " *******************************************************************"
  echo " *                                                                 *"
  echo " *           The Target BIOS version is already installed.         *"
  echo " *            Press 'q' to abort and any key to continue           *"
  echo " *                                                                 *"
  echo " *******************************************************************"
  echo " "
  pause > nul
  cls
endif

:UPDATE_START
# Check if according ME flash BIN file exists
set -v MeFlashFile "MEFW_K%product%.%BIOS_TARGET_VERSION%.BIN"
if not exist %MeFlashFile% then
  goto ERR_ME_FILE
endif

# Check ROM ID from flashfile and compare it with product name of the board
%AFU% %FlashFile% /U >a afu.out
%KoFind% /M /D" K" "ROM File ROM ID = K*-??" afu.out > nul
if %product% neq %FindStr% then
  set -d FindStr > nul
  goto ERR_ROMID
endif

#Show message to user
:EXECUTE
echo " "
echo " > BIOS for motherboard  : K%product%"
echo " > BIOS version installed: %BIOSV%"
echo " > BIOS flash file name  : %FlashFile%"
echo " > MEFW flash file name  : %MeFlashFile%"
echo " "
  echo " *******************************************************************"
  echo " *                                                                 *"
echo " *         Are you sure to update the ME and BIOS Firmware?        *"
  echo " *            Press 'q' to abort and any key to continue           *"
  echo " *                                                                 *"
  echo " *******************************************************************"
  echo " "
  pause > nul

:FLASH_ME
%FWUpdLcl% %FWUpdLclParams% %MeFlashFile%
set -v FWUpdLclERR %lasterror%
if %FWUpdLclERR% neq "0x0" then
  if %FWUpdLclERR% eq "0x1C0B" then
    # ME is inactive, perhaps by enabled BIOS option "EC Firmware Update"
    goto ERR_MEinactive
  endif
  # Ignore errorlevel 0x14B0C (Firmware version is already installed)
  if %FWUpdLclERR% eq "0x14B0C" then
    cls
    echo " "
    echo " ME firmware is already up to date. Continue with BIOS firmware update..."
    echo " "
    goto FLASH_START
  endif
  # Ignore errorlevel 0x16525 (Update to Image with lower VCN not allowed)
  if %FWUpdLclERR% eq "0x16525" then
    cls
    echo " "
    echo " ME firmware flash is not allowed due to lower VCN in file image."
    echo " Continue with BIOS firmware update..."
    echo " "
    goto FLASH_START
  else
    goto ERR_FWUpdLcl
  endif
endif

:FLASH_START
%AFU% %FlashFile% %AFUParams%
set -v AFUERR %lasterror%
if %AFUERR% neq "0x0" then
  goto ERR_AFU
endif 

:FLASH_DONE
rm smbios.out > nul
rm afu.out > nul
set -d FlashFile > nul
set -d product > nul
set -d FindStr > nul
set -d ME_flashed > nul

# Automatic restart...
cls 0 10
echo " "
echo " *******************************************************************"
echo " *  DONE: Flash successfull. System has to be restarted!           *"
echo " *                                                                 *"
echo " *  HINT: In general a power cycle is suggested...                 *"
echo " *          1. Wait until the system switched off                  *"
echo " *          2. Remove AC cable                                     *"
echo " *          3. Wait until all voltage rails are discharged         *"
echo " *          4. Re-connect AC cable an switch system on             *"
echo " *                                                                 *"
echo " *  Press 'q' to abort or any key to shutdown now.                 *"
echo " *                                                                 *"
echo " *******************************************************************"
echo " "
pause > nul
reset -s
goto EOF

:ERR_FILE
echo " "
echo " *******************************************************************"
echo " *   ERROR: Flash file and board does not match!                   *"
echo " *******************************************************************"
echo " "
goto EOF      

:ERR_ME_FILE
echo " "
echo " *******************************************************************"
echo " *   ERROR: ME Flash file not found!                               *"
echo " *******************************************************************"
echo " "
goto EOF      

:ERR_ROMID
echo " "
echo " *******************************************************************"
echo " *   ERROR: ROM ID from Flashfile does not match to system!        *"
echo " *******************************************************************"
echo " "
goto EOF 

:ERR_AFU
echo " "
echo " *******************************************************************"
echo " *   ERROR: AFU flash tool reported an error.                      *"
echo " *          Please check and try again.                            *"
echo " *******************************************************************"
echo " "
echo " Returncode: %AFUERR%"
echo " "
goto EOF   

:ERR_FWUpdLcl
echo " "
echo " *******************************************************************"
echo " *   ERROR: FWUpdLcl reported an error.                            *"
echo " *          Please check and try again.                            *"
echo " *******************************************************************"
echo " "
echo " Returncode: %FWUpdLclERR%"
echo " "
goto EOF   

:ERR_MEinactive
echo " "
echo " *******************************************************************"
echo " *   ERROR: ME is not running.                                     *"
echo " *          Please reboot and re-run the script afterwards.        *"
echo " *                                                                 *"
echo " *          Press 'q' to abort or any key to reboot                *"
echo " *                                                                 *"
echo " *******************************************************************"
echo " "
pause > nul
reset
goto EOF

:EOF
